Option Explicit

Sub DescribeError (ByVal hdbc As Long, ByVal hstmt As Long)
    ' Print an error message for the given connection handle
    ' and statement handle
    Dim sSQLState As String * 16
    Dim sErrorMsg As String * 256
    Dim iMsgLen As Integer
    Dim lNative As Long
    Dim iRC As Integer
    
    ' Process all pending errors...
    Do
	iRC = SQLError(0, hdbc, hstmt, sSQLState, lNative, sErrorMsg, 256, iMsgLen)
	If iRC = SQL_SUCCESS Or iRC = SQL_SUCCESS_WITH_INFO Then
	    If iMsgLen = 0 Then
		MsgBox "Error -- No error information available", 0, "SQL Error"
	    Else
		MsgBox Format$(lNative) & " - " & StripNullStrings(sSQLState) & " - " & Left$(sErrorMsg, iMsgLen), 0, "SQL Error"
	    End If
	End If
    Loop Until iRC <> SQL_SUCCESS
End Sub

Function GetODBCWorkString (hdbc As Long, hstmt As Long, i As Integer) As String
Dim nStatus As Integer
Dim nDataType As Integer
Dim nNullable As Integer
Dim nScale As Integer
Dim lPrecision As Long
Dim lOutputBufferLength As Long
Dim sLongDataBuffer As String * MAX_LONG_DATA_BUFFER
Dim workString As String

nStatus = SQLDescribeCol(hstmt, i, sLongDataBuffer, Len(sLongDataBuffer), MAX_LONG_DATA_BUFFER, nDataType, lPrecision, nScale, nNullable)
If nStatus <> SQL_SUCCESS Then
    DescribeError hdbc, hstmt
    GetODBCWorkString = "Null"
    Exit Function
End If
sLongDataBuffer = Space$(0)
nStatus = SQLGetData(hstmt, i, SQL_C_CHAR, sLongDataBuffer, MAX_LONG_DATA_BUFFER, lOutputBufferLength)
If nStatus = SQL_SUCCESS Or nStatus = SQL_SUCCESS_WITH_INFO Then
Else
    If nStatus <> SQL_SUCCESS Then
	DescribeError hdbc, hstmt
	GetODBCWorkString = "Null"
	Exit Function
    End If
End If

If lOutputBufferLength = -1 Then
    workString = ""
Else
    workString = Left$(sLongDataBuffer, lOutputBufferLength)
End If

Select Case nDataType
Case SQL_INTEGER
    If workString <> "" Then
'        workString = Format$(Val(RTrim(workString)), "0;0-;0;#")
	workString = Format$(workString, "0;-0;0;#")
    End If
Case SQL_NUMERIC
    If workString <> "" Then
'        workString = Format$(Val(RTrim(workString)), "0;0-;0;#")
	If nScale <> 0 Then
	    workString = Format$(workString, "0.00;-0.00;0.00;#")
	Else
	    workString = Format$(workString, "0;-0;0;#")
	End If
    End If
Case SQL_DECIMAL
    If workString <> "" Then
'        workString = Format$(Val(RTrim(workString)), "0.00;0.00-;0.00;#")
	If nScale <> 0 Then
	    workString = Format$(workString, "0.00;-0.00;0.00;#")
	Else
	    workString = Format$(workString, "0;-0;0;#")
	End If
    End If
Case SQL_DOUBLE
    If workString <> "" Then
'        workString = Format$(Val(RTrim(workString)), "0.00;0.00-;0.00;#")
	If nScale <> 0 Then
	    workString = Format$(workString, "0.00;-0.00;0.00;#")
	Else
	    workString = Format$(workString, "0;-0;0;#")
	End If
    End If
Case SQL_SMALLINT
    If workString <> "" Then
'        workString = Format$(Val(RTrim(workString)), "0;0-;0;#")
	workString = Format$(workString, "0;-0;0;#")
    End If
Case SQL_FLOAT
    If workString <> "" Then
'        workString = Format$(Val(RTrim(workString)), "0.00;0.00-;0.00;#")
	If nScale <> 0 Then
	    workString = Format$(workString, "0.00;-0.00;0.00;#")
	Else
	    workString = Format$(workString, "0;-0;0;#")
	End If
    End If
Case SQL_TIMESTAMP
    If workString <> "" Then
	workString = Mid$(workString, 6, 2) & "/" & Mid$(workString, 9, 2) & "/" & Left$(workString, 4)
    End If
    If workString = "01/01/1900" Then
	workString = ""
    End If
End Select
'
GetODBCWorkString = RTrim$(workString)
'
End Function

Sub InitializeDatabaseConnection ()
Dim nBufferSizeBack As Integer
Dim nStatus As Integer
Dim startPos As Integer
Dim endPos As Integer
Dim lConnectTimeout As Integer
Dim nCounter As Integer

nStatus = SQLAllocEnv(ghenv)
If nStatus <> SQL_SUCCESS Then
    DescribeError ghenv, 0
    End
End If
   
'*** Connection string for ODBC Logon
If gsConnectString > " " Then
Else
    gsConnectString = "DSN=" & gsDataSourceName & ";DATABASE=" & gsDatabase & ";UID=" & gsUserName & ";WSID=CTS100v1.0"

End If
'*** The following three calls allocate environment space and log
'*** the user onto the server for connection 1

nStatus = SQLAllocConnect(ghenv, ghdbc)
If nStatus <> SQL_SUCCESS Then
    DescribeError ghdbc, 0
    Exit Sub
End If
'
lConnectTimeout = 5
'
'
nStatus = SQLSetConnectOption(ghdbc, SQL_LOGIN_TIMEOUT, lConnectTimeout)
Do
    nStatus = SQLDriverConnect(ghdbc, form1.hWnd, gsConnectString, Len(gsConnectString), gsResultString, Len(gsResultString), nBufferSizeBack, 1)
    DoEvents
    If nStatus = SQL_NO_DATA_FOUND Then
	MsgBox "Logon cancelled.", MB_ICONSTOP, "ODBC Encapsulation"
	nStatus = SQLFreeConnect(ghdbc)
	nStatus = SQLFreeEnv(ghenv)
	End
    Else
	If nStatus <> SQL_SUCCESS And nStatus <> SQL_SUCCESS_WITH_INFO Then
	    DoEvents
	    nCounter = nCounter + 1
	    If nCounter > 3 Then
		DescribeError ghdbc, 0
		Exit Do
	    End If
	    nStatus = SQLFreeConnect(ghdbc)
	    nStatus = SQLAllocConnect(ghenv, ghdbc)
	    If nStatus <> SQL_SUCCESS Then
		DescribeError ghdbc, 0
		Exit Sub
	    End If
	    lConnectTimeout = lConnectTimeout * 2
	    nStatus = SQLSetConnectOption(ghdbc, SQL_LOGIN_TIMEOUT, lConnectTimeout)
	    DoEvents
	Else
	    Exit Do
	End If
    End If
Loop
'
gsConnectString = gsResultString

startPos = InStr(1, gsConnectString, "UID=") + 4
endPos = InStr(startPos, gsConnectString, ";")
gsUserName = Mid$(gsConnectString, startPos, (endPos - startPos))

End Sub

Sub InitializeStatementHandle ()
Dim nStatus As Integer
    
nStatus = SQLAllocStmt(ghdbc, ghstmt)
If nStatus <> SQL_SUCCESS And nStatus <> SQL_SUCCESS_WITH_INFO Then
    DescribeError ghdbc, ghstmt
    Exit Sub
End If
nStatus = SQLSetStmtOption(ghstmt, SQL_QUERY_TIMEOUT, 60)
If nStatus <> SQL_SUCCESS Then
    DescribeError ghdbc, ghstmt
End If

End Sub

